@@ -2,14 +2,17 @@
 import re
 import copy
 import logging
+from time import sleep
 from notification.models import TaskHistory
 from dbaas_aclapi import helpers
 from dbaas_aclapi.acl_base_client import get_acl_client
-from acl_base_client import STATUS_PENDING
+from acl_base_client import STATUS_PENDING, STATUS_ERROR
 
 logging.basicConfig()
 LOG = logging.getLogger("AclTask")
 LOG.setLevel(logging.DEBUG)
+MAX_RETRIES_CHECK_APPLY = 25
+SLEEP_CHECK_APPLY = 4
 
 
 class ReplicateACLError(Exception):
@@ -19,7 +22,6 @@ class ReplicateACLError(Exception):
 class ReplicateACLAllErrors(Exception):
     pass
 
-
 class RunJobError(Exception):
     pass
 
@@ -65,6 +67,7 @@ def replicate_acl_for(database, old_ip, new_ip, old_sa=None, new_sa=None):
         # get and apply pending acls.
         if acl.is_libera_3:
             for dest in [new_ip, new_sa]:
+                current_apply_try = 0
                 if not dest:
                     continue
 
@@ -72,11 +75,45 @@ def replicate_acl_for(database, old_ip, new_ip, old_sa=None, new_sa=None):
                 if is_ip(search_destination):
                     search_destination = "%s/32" % search_destination
 
-                for rule in helpers.iter_on_acl_rules(
-                 acl,
-                 {"destination": search_destination,
-                  "status": STATUS_PENDING}):
-                    acl.apply_acl(rule)
+                search_pending = {
+                    "destination": search_destination,
+                    "status": STATUS_PENDING
+                }
+                while True:
+                    has_pending = False
+                    to_apply = helpers.iter_on_acl_rules(
+                                acl,
+                                search_pending)
+
+                    for rule in to_apply:
+                        has_pending = True
+                        acl.apply_acl(rule)
+
+                    if not has_pending:
+                        break
+
+                    if current_apply_try > MAX_RETRIES_CHECK_APPLY:
+                        for rule in to_apply:
+                            all_errors.append(
+                                "Error while apply ACLs. Rule {} are pending".format(rule)
+                            )
+                        break
+
+                    current_apply_try += 1
+                    sleep(SLEEP_CHECK_APPLY)
+
+                search_errors = {
+                    "destination": search_destination,
+                    "status": STATUS_ERROR
+                }
+                has_error = helpers.iter_on_acl_rules(
+                                acl,
+                                search_errors)
+
+                for rule in has_error:
+                    all_errors.append(
+                        "Rule {} has ERROR status.".format(rule)
+                    )
 
         if all_errors:
             raise ReplicateACLAllErrors(
